@@ -15,6 +15,12 @@ struct MyOtherStruct {
   s: &'static str,
 }
 
+impl Clone for MyOtherStruct {
+  fn clone(&self) -> Self {
+    Self { s: self.s }
+  }
+}
+
 impl Drop for MyOtherStruct {
   fn drop(&mut self) {
     println!("Dropping MyOtherStruct with value: {}", self.s)
@@ -365,4 +371,30 @@ fn main() {
   v.push(Box::new(MyOtherStruct { s: "BBB" }));
   v.push(Box::new(MyOtherStruct { s: "CCC" }));
   println!("{} {}", v.capacity(), v.len());
+  let cc = staticvec!["AAA", "BBB"];
+  let dd = cc.clone();
+  let mut ee = StaticVec::new();
+  ee.clone_from(&dd);
+  for s in &ee.reversed() {
+    println!("{}", s);
+  }
+  let clonee = staticvec![
+    Box::new(MyOtherStruct { s: "AAA" }),
+    Box::new(MyOtherStruct { s: "BBB" }),
+    Box::new(MyOtherStruct { s: "CCC" }),
+    Box::new(MyOtherStruct { s: "DDD" }),
+    Box::new(MyOtherStruct { s: "EEE" }),
+  ];
+  let cloned = clonee.clone();
+  for ms in &cloned {
+    println!("{:?}", ms);
+  }
+  let v2 = StaticVec::from([
+    Box::new(MyOtherStruct { s: "XYZ" }),
+    Box::new(MyOtherStruct { s: "ZYX" }),
+    Box::new(MyOtherStruct { s: "XYZ" }),
+  ]);
+  let mut it = v2.into_iter();
+  println!("{:?}", it.next().unwrap());
+  println!("{:?}", it.next().unwrap());
 }
